module Booth

#(
	// Parameter Declarations
	parameter n = 16
)

(
	// Input Ports
	// Multiplicando r y Multiplicador m
	input [n-1:0] r,
	input [n-1:0] m,	
	// reloj clk y senal start	
	input clk, start,

	// Output Ports
	// Producto result
	output [2*n-1:0] result,
	// salida done
	output done
);

// Variables	
reg [2*n-1:0] reg_p;
reg [n-1:0] reg_a;
reg [n-1:0] reg_b; 
reg low_m;
reg DFF0;
reg donei;
reg [n-1:0] suma;
integer iteracion;
reg [n-1:0] out_mux2;
reg [n-1:0] out_mux3;

// Nodos
wire init;
wire cout;

// Logica de DFF0
always@(posedge clk)
begin
	DFF0 <= start;
end
// Logica init
assign init = !DFF0 && start;
	
// mux0 + reg_a
always@(posedge clk)
begin
	if (init == 1'b1) 
		reg_a <= r;
	else 
		reg_a <= reg_a;
end

// mux1 + reg_b + low_m
always@(posedge clk)
begin
	if (init == 1'b1) begin
		reg_b <= m;
		low_m <= 1'b0;
		end
	else begin
		reg_b <= {reg_b[n-1],reg_b[n-1:1]} ;
		low_m <= reg_b[0];
		end
end

// mux2
always@(reg_a or reg_b or low_m)
begin
	if (reg_b[0] ^ low_m) out_mux2 = 'b0;
	else out_mux2 = reg_a;
end

// Sumador Restador
always@(*)
begin
	if (!reg_b && low_m) suma = reg_p[2*n-1 : n] - out_mux2;
	else suma = reg_p[2*n-1 : n] + out_mux2;
end

//mux3
always@(*)
begin
 if (init) out_mux3 = 'b0;
 else out_mux3 = {cout, suma[n-1:1]};
end

assign cout = reg_p[2*n-1] && !suma[n-1] || out_mux2[n-1] && !suma[n-1] || reg_p[2*n-1] && out_mux2[n-1] ;

// mux5 y reg_p
always@(posedge clk)
begin
	if (done) reg_p <= reg_p;
	else reg_p <= {out_mux3[n-1],out_mux3[n-1:0],reg_p[n-1:1]};
end
assign result = reg_p;

// Contador de iteraciones
always@(posedge clk)
begin
	if (init) iteracion = 0;
	else if(!done) iteracion <= iteracion + 1;
end

always@(*)
begin
	if (iteracion >= n) donei = 1'b1;
	else donei = 1'b0;
end
assign done = donei;
endmodule
